#ifndef __String_h__
#define __String_h__

#include "Vector.h"
#include <cstring>
#include <iostream>
#include <string>

class String
{
public:
	typedef std::string Super;

	String( Super super ) : _super( super )
	{
	}

	String()
	{
	}

    String( const char *string, unsigned length ) : _super( string, length )
    {
    }

	String( const char *string ) : _super( string )
	{
	}

	unsigned length() const
	{
		return _super.length();
	}

	const char *ascii() const
	{
		return _super.c_str();
	}

	char operator[]( int index )
	{
		return _super[index];
	}

	String &operator=( const char *string )
	{
		*this = String( string );
		return *this;
	}

	String &operator=( const String &other )
	{
		_super = other._super;
		return *this;
	}

	bool operator==( const char *string ) const
	{
		return _super.compare( string ) == 0;
	}

	String operator+( const String &other ) const
	{
		return _super + other._super;
	}

	String operator+( const char *other ) const
	{
		return _super + other;
	}

	String operator+=( const String &other )
	{
		return _super += other._super;
	}

	String operator+=( const char *other )
	{
		return _super += other;
	}

	bool operator==( const String &other ) const
	{
        return _super.compare( other._super ) == 0;
	}

    bool operator!=( const String &other ) const
    {
        return _super.compare( other._super ) != 0;
    }

    bool operator<( const String &other ) const
    {
        return _super < other._super;
    }

    Vector<String> tokenize( const String &delimiters ) const
    {
        Vector<String> tokens;

        char copy[length() + 1];
        strcpy( copy, ascii() );

        char delimiterCopy[delimiters.length() + 1];
        strcpy( delimiterCopy, delimiters.ascii() );

        char *token = strtok( copy, delimiterCopy );

        while ( token != NULL )
        {
            tokens.append( String( token ) );
            token = strtok( NULL, delimiterCopy );
        }

        return tokens;
    }

    bool contains( const String &substring ) const
    {
        return find( substring ) != Super::npos;
    }

    unsigned find( const String &substring ) const
    {
        return _super.find( substring._super );
    }

    String substring( unsigned fromIndex, unsigned howMany ) const
    {
        return _super.substr( fromIndex, howMany );
    }

    bool endsWith( const String &suffix ) const
    {
        return substring( length() - suffix.length(), suffix.length() ) == suffix;
    }

    void replace( const String &toReplace, const String &replaceWith )
    {
        if ( find( toReplace ) != Super::npos )
        {
            _super.replace( find( toReplace ), toReplace.length(), replaceWith._super );
        }
    }

    String trim()
    {
        unsigned firstNonSpace = 0;
        unsigned lastNonSpace = 0;

        bool firstNonSpaceFound = false;

        for ( unsigned i = 0; i < length(); ++i )
        {
            if ( ( !firstNonSpaceFound ) && ( _super[i] != ' ' ) )
            {
                firstNonSpace = i;
                firstNonSpaceFound = true;
            }

            if ( ( _super[i] != ' ' ) && ( _super[i] != '\n' ) )
            {
                lastNonSpace = i;
            }
        }

        if ( lastNonSpace == 0 )
        {
            return "";
        }

        return substring( firstNonSpace, lastNonSpace - firstNonSpace + 1 );
    }

    friend std::istream &operator>>( std::istream &stream, String &string );

protected:
	Super _super;
};

// #ifdef CXXTEST_RUNNING
// namespace CxxTest { CXXTEST_COPY_TRAITS( String, std::string ); }
// #endif // CXXTEST_RUNNING

#endif // __String_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
