#include <cxxtest/TestSuite.h>

#include "Error.h"
#include "MockErrno.h"
#include "String.h"
#include "Bitset.h"

class BitsetTestSuite : public CxxTest::TestSuite
{
public:
    MockErrno *mockErrno;

    void setUp()
    {
        TS_ASSERT( mockErrno = new MockErrno );
    }

    void tearDown()
    {
        TS_ASSERT_THROWS_NOTHING( delete mockErrno );
    }

    void test_set_get()
    {
        Bitset<3> bitset;

        bitset.set( 0 );
        bitset.set( 2 );

        TS_ASSERT( bitset.get( 0 ) );
        TS_ASSERT( !bitset.get( 1 ) );
        TS_ASSERT( bitset.get( 2 ) );
    }

    void test_reset()
    {
        Bitset<3> bitset;

        bitset.set( 0 );
        bitset.set( 2 );

        TS_ASSERT( bitset.get( 0 ) );
        TS_ASSERT( !bitset.get( 1 ) );
        TS_ASSERT( bitset.get( 2 ) );

        TS_ASSERT_THROWS_NOTHING( bitset.clear() );

        TS_ASSERT( !bitset.get( 0 ) );
        TS_ASSERT( !bitset.get( 1 ) );
        TS_ASSERT( !bitset.get( 2 ) );

        bitset.set( 0 );
        bitset.set( 2 );

        TS_ASSERT( bitset.get( 0 ) );
        TS_ASSERT( !bitset.get( 1 ) );
        TS_ASSERT( bitset.get( 2 ) );
    }
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
