#include <cxxtest/TestSuite.h>

#include "Error.h"
#include "MockErrno.h"
#include "String.h"
#include "List.h"

class ListTestSuite : public CxxTest::TestSuite
{
public:
    MockErrno *mockErrno;

    void setUp()
    {
        TS_ASSERT( mockErrno = new MockErrno );
    }

    void tearDown()
    {
        TS_ASSERT_THROWS_NOTHING( delete mockErrno );
    }

    void test_insert_and_erase()
    {
        List<unsigned> list;

        TS_ASSERT( list.empty() );
        TS_ASSERT_EQUALS( list.size(), 0U );

        List<unsigned>::iterator it = list.begin();

        TS_ASSERT_EQUALS( it, list.end() );

        it = list.insert( it, 5 );

        TS_ASSERT_EQUALS( *it, 5U );
        TS_ASSERT_EQUALS( list.size(), 1U );

        it = list.insert( it, 3 );

        TS_ASSERT_EQUALS( *it, 3U );
        TS_ASSERT_EQUALS( list.size(), 2U );

        ++it;
        TS_ASSERT_EQUALS( *it, 5U );

        ++it;
        TS_ASSERT_EQUALS( it, list.end() );

        it = list.begin();
        it = list.erase( it );
        TS_ASSERT_EQUALS( *it, 5U );
        TS_ASSERT_EQUALS( list.size(), 1U );

        it = list.erase( it );
        TS_ASSERT_EQUALS( it, list.end() );
        TS_ASSERT( list.empty() );
    }

    void test_erase_element()
    {
        List<unsigned> list;

        TS_ASSERT( list.empty() );
        TS_ASSERT_EQUALS( list.size(), 0U );

        List<unsigned>::iterator it = list.begin();

        TS_ASSERT_EQUALS( it, list.end() );

        it = list.insert( it, 5 );
        ++it;
        it = list.insert( it, 3 );
        ++it;
        it = list.insert( it, 6 );
        ++it;
        it = list.insert( it, 3 );

        TS_ASSERT_EQUALS( list.size(), 4U );

        TS_ASSERT( list.exists( 3 ) );
        TS_ASSERT( list.exists( 5 ) );
        TS_ASSERT( !list.exists( 4 ) );

        TS_ASSERT_THROWS_NOTHING( list.eraseByValue( 3 ) );

        TS_ASSERT( list.exists( 3 ) );

        TS_ASSERT_EQUALS( list.size(), 3U );

        it = list.begin();

        TS_ASSERT_EQUALS( *it, 5U );
        ++it;
        TS_ASSERT_EQUALS( *it, 6U );
        ++it;
        TS_ASSERT_EQUALS( *it, 3U );
        ++it;
        TS_ASSERT_EQUALS( it, list.end() );

        TS_ASSERT_THROWS_NOTHING( list.eraseByValue( 3 ) );

        TS_ASSERT( !list.exists( 3 ) );

        TS_ASSERT_EQUALS( list.size(), 2U );

        it = list.begin();

        TS_ASSERT_EQUALS( *it, 5U );
        ++it;
        TS_ASSERT_EQUALS( *it, 6U );
        ++it;
        TS_ASSERT_EQUALS( it, list.end() );
    }


    void test_append()
    {
        List<int> list;

        TS_ASSERT_EQUALS( list.begin(), list.end() );

        list.append( 5 );
        list.append( 8 );

        List<int>::iterator it = list.begin();

        TS_ASSERT_EQUALS( *it, 5 );
        ++it;
        TS_ASSERT_EQUALS( *it, 8 );
        ++it;
        TS_ASSERT_EQUALS( it, list.end() );

        --it;
        TS_ASSERT_EQUALS( *it, 8 );
        --it;
        TS_ASSERT_EQUALS( *it, 5 );
        TS_ASSERT_EQUALS( it, list.begin() );

        list.appendHead( 3 );
        TS_ASSERT_DIFFERS( it, list.begin() );
        --it;
        TS_ASSERT_EQUALS( it, list.begin() );
        TS_ASSERT_EQUALS( *it, 3 );
    }

    void test_equality_operator()
    {
        List<int> list1, list2;

        TS_ASSERT_EQUALS( list1, list2 );

        list1.append( 1 );
        TS_ASSERT_DIFFERS( list1, list2 );

        list2.append( 1 );
        TS_ASSERT_EQUALS( list1, list2 );

        list1.append( 2 );
        list1.append( 3 );

        list2.append( 3 );
        list2.append( 2 );

        TS_ASSERT_DIFFERS( list1, list2 );
    }
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
