#ifndef __Engine_h__
#define __Engine_h__

#include "AutoRegularThreadListener.h"
#include "AutoNonBlockingThreadListener.h"
#include "AutoSelector.h"
#include "AutoWeaver.h"
#include "EngineConfiguration.h"
#include "Error.h"
#include "IEngine.h"
#include "T/Truth.h"

class Engine : public IEngine
{
public:
	static void main()
	{
		Engine().entryPoint();
	}

	Engine() : _weaver( _selector ), _regularThreadListener( _selector, _weaver ), _nonBlockingThreadListener( _selector, _weaver )
	{
	}

	void entryPoint()
	{
        try
        {
            while ( T::alwaysTrue() )
            {
                _selector->select();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: Engine" );
            throw e;
        }
    }

private:
	AutoSelector _selector;
	AutoWeaver _weaver;
    AutoRegularThreadListener _regularThreadListener;
    AutoNonBlockingThreadListener _nonBlockingThreadListener;
};

#endif // __Engine_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
