#ifndef __AckMaster_h__
#define __AckMaster_h__

#include "AckNumberPayload.h"
#include "BThread.h"
#include "ConnectionIdentifier.h"
#include "IEvent.h"
#include "Map.h"
#include "TcpEvent.h"

class AckMaster : public BThread
{
public:
    AckMaster() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForRequests();
                storeRequest();

                if ( setAckNumberRequest() )
                {
                    setAckNumber();
                }
                else
                {
                    sendAckNumber();
                }
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: RawSocketSender" );
            throw e;
        }
    }

private:
    unsigned _nextAckNumber;
    unsigned _sizeOfNextPacket;
    Map<ConnectionIdentifier, unsigned> _connectionToAckNumber;
    TcpEvent _request;

    void waitForRequests()
    {
        clearEventVectors();

        _watched.append( IEvent::SET_ACK_NUMBER );
        _watched.append( IEvent::ACK_NUMBER_REQUEST );

        _blocked.append( IEvent::ACK_NUMBER_REPLY );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void storeRequest()
    {
        _request.constructFromEvent( lastEvent() );
    }

    bool setAckNumberRequest() const
    {
        return lastEvent().type() == IEvent::SET_ACK_NUMBER;
    }

    void setAckNumber()
    {
        ConnectionIdentifier connectionIdentifier( _request.getIp(), _request.getPort() );
        _connectionToAckNumber[connectionIdentifier] = AckNumberPayload( _request.getPacketPayload() ).getAckNumber();
    }

    void sendAckNumber()
    {
        clearEventVectors();

        _blocked.append( IEvent::SET_ACK_NUMBER );
        _blocked.append( IEvent::ACK_NUMBER_REQUEST );

        ConnectionIdentifier connectionIdentifier( _request.getIp(), _request.getPort() );
        AckNumberPayload payload( _connectionToAckNumber[connectionIdentifier] );
        _requested.append( TcpEvent( IEvent::ACK_NUMBER_REPLY, _request.getIp(), _request.getPort(), ConstSimpleData( &payload, sizeof(payload) ) ) );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }
};

#endif // __AckMaster_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
