#ifndef __HttpFileReceiver_h__
#define __HttpFileReceiver_h__

#include "BThread.h"
#include "ConnectionIdentifier.h"
#include "File.h"
#include "IEvent.h"
#include "ServerConfiguration.h"

class HttpFileReceiver : public BThread
{
public:
    HttpFileReceiver() : BThread( BThread::REGULAR_THREAD ), _incomingFileIndex( 1 )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForRequests();
                storeRequest();
                extractIdentifier();
                openFileForWriting();
                dumpData();
                closeFile();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: HttpFileReceiver" );
            throw e;
        }
    }

private:
    TcpEvent _incomingPacket;
    unsigned _incomingFileIndex;
    IFile *_file;
    String _fileName;
    ConnectionIdentifier _connectionIdentifier;
    Map<ConnectionIdentifier, unsigned> _connectionToIndex;

    void waitForRequests()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_DATA_RECEIVED );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void storeRequest()
    {
        _incomingPacket.constructFromEvent( lastEvent() );
    }

    void extractIdentifier()
    {
        _connectionIdentifier = ConnectionIdentifier( _incomingPacket.getIp(), _incomingPacket.getPort() );
    }

    void openFileForWriting()
    {
        chooseFileName();
        openFile();
    }

    void chooseFileName()
    {
        if ( !_connectionToIndex.exists( _connectionIdentifier ) )
        {
            _connectionToIndex[_connectionIdentifier] = _incomingFileIndex;
            ++_incomingFileIndex;
        }

        _fileName = String( ServerConfiguration::INCOMING_FILE_DIR ) + "/" + Stringf( "file%u", _connectionToIndex[_connectionIdentifier] );
    }

    void openFile()
    {
        _file = new File( _fileName );
        _file->open( IFile::MODE_APPEND );
    }

    void dumpData()
    {
        _file->write( _incomingPacket.getPacketPayload() );
    }

    void closeFile()
    {
        delete _file;
        _file = NULL;
    }
};

#endif // __HttpFileReceiver_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
