#ifndef __HttpFoundHandler_h__
#define __HttpFoundHandler_h__

#include "BThread.h"
#include "File.h"
#include "IEvent.h"

class HttpFoundHandler : public BThread
{
public:
    HttpFoundHandler() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForFoundRequest();
                storeRequest();
                sendResponse();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: HttpFoundHandler" );
            throw e;
        }
    }

private:
    enum {
        MAX_FILE_SIZE = 4096,
    };

    TcpEvent _foundRequest;

    void waitForFoundRequest()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_GENERATE_FOUND );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void storeRequest()
    {
        _foundRequest.constructFromEvent( lastEvent() );
    }

    void sendResponse()
    {
        Stringf foundString( "%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
                             "HTTP/1.1 302 Found\r\n",
                             "Server: BP Server\r\n",
                             "Content-Type: text/html; charset=\%s\r\n",
                             "Location: ",
                             properLocation().ascii(),
                             "\r\nConnection: close\r\n",
                             "\r\n",
                             "<HTML>\n",
                             "<HEAD><TITLE>302 Found Found</TITLE></HEAD>\n",
                             "<BODY>\n",
                             "<H4>302 Found</H4>\n",
                             "Directories must end with a slash.\n",
                             "</BODY>\n",
                             "</HTML>\n" );

        HeapData foundMessage = ConstSimpleData( foundString.ascii(), foundString.length() );

        clearEventVectors();
        _requested.append( TcpEvent( IEvent::DATA_FROM_HTTP_TO_TCP, _foundRequest.getIp(), _foundRequest.getPort(), foundMessage ) );
        _blocked.append( IEvent::HTTP_GENERATE_FOUND );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    String properLocation()
    {
        String original( _foundRequest.getPacketPayload().asChar(), _foundRequest.getPacketPayload().size() );
        return original + "/";
    }
};

#endif // __HttpFoundHandler_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
