#ifndef __IEvent_h__
#define __IEvent_h__

#include "Event.h"
#include "Stringf.h"

class IEvent
{
public:
    enum EventType {
        HTTP_DATA_RECEIVED = 0,

        HTTP_GENERATE_ERROR = 1,
        HTTP_GENERATE_FOUND = 2,
        HTTP_GET_REQUEST = 3,

        TCP_SEGMENT_RECEIVED = 4,
        TCP_DONE_HADNLING_ACK = 5,

        TCP_FIN_REQUEST = 6,
        TCP_PUSH_REQUEST = 7,
        TCP_SEND_ACK = 8,
        TCP_OUTGOING_SEGMENT = 9,
        TCP_SEND_PUSH = 10,
        TCP_SYN_REQUEST = 11,

        SYN_HANDLER_REQUEST_SEQUENCE_NUMBER = 12,
        SEQUENCE_NUMBER_FOR_SYN_HANDLER = 13,

        FIN_HANDLER_REQUEST_SEQUENCE_NUMBER = 14,
        SEQUENCE_NUMBER_FOR_FIN_HANDLER = 15,

        ACK_SENDER_REQUEST_SEQUENCE_NUMBER = 16,
        SEQUENCE_NUMBER_FOR_ACK_SENDER = 17,

        PUSH_SENDER_REQUEST_SEQUENCE_NUMBER = 18,
        SEQUENCE_NUMBER_FOR_PUSH_SENDER = 19,

        DATA_FROM_TCP_TO_HTTP = 20,
        DATA_FROM_HTTP_TO_TCP = 21,

        CONNECTION_TIMEOUT_DETECTED = 22,

        IDLE_EVENT = 23,

        TCP_SEND_FIN = 24,

        FIN_SENDER_REQUEST_SEQUENCE_NUMBER = 25,
        SEQUENCE_NUMBER_FOR_FIN_SENDER = 26,

        KEEPALIVE_SENDER_REQUEST_SEQUENCE_NUMBER = 27,
        SEQUENCE_NUMBER_FOR_KEEPALIVE_SENDER = 28,

        SET_ACK_NUMBER = 29,
        ACK_NUMBER_REQUEST = 30,
        ACK_NUMBER_REPLY = 31,

        TCP_ACK_REQUEST = 32,
        TCP_STORE_PACKET = 33,

        ORDERED_DATA_FROM_HTTP_TO_TCP = 34,

        HTTP_GET_CGI_REQUEST = 35,

        TCP_SEND_SEGMENT = 36,

        NUMBER_OF_EVENTS = 37,
    };

    static Vector<Event> allEvents()
    {
        Vector<Event> result;

        for ( unsigned i = 0; i < NUMBER_OF_EVENTS; ++i )
        {
            result.append( Event( i ) );
        }

        return result;
    }

    static Vector<Event> allEventsExcept( IEvent::EventType eventType )
    {
        Vector<Event> result;

        for ( unsigned i = 0; i < NUMBER_OF_EVENTS; ++i )
        {
            if ( i != eventType )
            {
                result.append( Event( i ) );
            }
        }

        return result;
    }

    static String eventToString( Event &event )
    {

        switch ( event.type() )
        {
        case 0:
            return "HTTP_DATA_RECEIVED";
            break;
        case 1:
            return "HTTP_GENERATE_ERROR";
            break;
        case 2:
            return "HTTP_GENERATE_FOUND";
            break;
        case 3:
            return "HTTP_GET_REQUEST";
            break;
        case 4:
            return "TCP_SEGMENT_RECEIVED";
            break;
        case 5:
            return "TCP_DONE_HADNLING_ACK";
            break;
        case 6:
            return "TCP_FIN_REQUEST";
            break;
        case 7:
            return "TCP_PUSH_REQUEST";
            break;
        case 8:
            return "TCP_SEND_ACK";
            break;
        case 9:
            return "TCP_OUTGOING_SEGMENT";
            break;
        case 10:
            return "TCP_SEND_PUSH";
            break;
        case 11:
            return "TCP_SYN_REQUEST";
            break;
        case 12:
            return "SYN_HANDLER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 13:
            return "SEQUENCE_NUMBER_FOR_SYN_HANDLER";
            break;
        case 14:
            return "FIN_HANDLER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 15:
            return "SEQUENCE_NUMBER_FOR_FIN_HANDLER";
            break;
        case 16:
            return "ACK_SENDER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 17:
            return "SEQUENCE_NUMBER_FOR_ACK_SENDER";
            break;
        case 18:
            return "PUSH_SENDER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 19:
            return "SEQUENCE_NUMBER_FOR_PUSH_SENDER";
            break;
        case 20:
            return "DATA_FROM_TCP_TO_HTTP";
            break;
        case 21:
            return "DATA_FROM_HTTP_TO_TCP";
            break;
        case 22:
            return "CONNECTION_TIMEOUT_DETECTED";
            break;
        case 23:
            return "IDLE_EVENT";
            break;
        case 24:
            return "TCP_SEND_FIN";
            break;
        case 25:
            return "FIN_SENDER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 26:
            return "SEQUENCE_NUMBER_FOR_FIN_SENDER";
            break;
        case 27:
            return "KEEPALIVE_SENDER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 28:
            return "SEQUENCE_NUMBER_FOR_KEEPALIVE_SENDER";
            break;
        case 29:
            return "SET_ACK_NUMBER";
            break;
        case 30:
            return "ACK_NUMBER_REQUEST";
            break;
        case 31:
            return "ACK_NUMBER_REPLY";
            break;
        case 32:
            return "TCP_ACK_REQUEST";
            break;
        case 33:
            return "TCP_STORE_PACKET";
            break;
        case 34:
            return "ORDERED_DATA_FROM_HTTP_TO_TCP";
            break;
        case 35:
            return "HTTP_GET_CGI_REQUEST";
            break;
        case 36:
            return "TCP_SEND_SEGMENT";
            break;
        default:
            break;
        }

        return "\t\tUNKNOWN";
    }
};

#endif // __IEvent_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
