#ifndef __IpEvent_h__
#define __IpEvent_h__

#include "Event.h"

class IpEvent : public Event
{
public:
    IpEvent()
    {
    }

    IpEvent( unsigned type, const String &ip, const ConstSimpleData &payload ) : Event( type ), _ip( ip ), _packetPayload( payload )
    {
        buildCompletePayload();
    }

    IpEvent( const IpEvent &other ) : Event( other.type(), other.payload() ),
                                      _ip( other.getIp() ),
                                      _packetPayload( other.getPacketPayload() )
    {
    }

    void constructFromEvent( const Event &event )
    {
        *((Event *)this) = event;

        unsigned ipLength;
        memcpy( &ipLength, payload().data(), sizeof(ipLength) );
        _ip = String( payload().asChar() + sizeof(ipLength), ipLength );
        _packetPayload = ConstSimpleData( payload().asChar() + sizeof(ipLength) + ipLength, payload().size() - sizeof(ipLength) - ipLength );
    }

    IpEvent &operator=( const IpEvent &other )
    {
        _type = other.type();
        _payload = other.payload();

        _ip = other.getIp();
        _packetPayload = other.getPacketPayload();

        return *this;
    }

    String getIp() const
    {
        return _ip;
    }

    ConstSimpleData getPacketPayload() const
    {
        return ConstSimpleData( _packetPayload.data(), _packetPayload.size() );
    }

private:
    String _ip;
    HeapData _packetPayload;

    void buildCompletePayload()
    {
        unsigned ipLength = _ip.length();
        _payload = ConstSimpleData( &ipLength, sizeof(ipLength) );
        _payload += ConstSimpleData( _ip.ascii(), ipLength );
        _payload += _packetPayload;
    }
};

#endif // __IpEvent_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
