#ifndef __OutgoingSegmentHandler_h__
#define __OutgoingSegmentHandler_h__

#include "BThread.h"
#include "IEvent.h"
#include "IpEvent.h"
#include "ServerConfiguration.h"
#include "TcpSegment.h"

class OutgoingSegmentHandler : public BThread
{
public:
    OutgoingSegmentHandler() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForSegment();
                requestSendingOfSegment();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: OutgoingSegmentHandler" );
            throw e;
        }
    }

private:
    void waitForSegment()
    {
        clearEventVectors();
        _watched.append( IEvent::TCP_OUTGOING_SEGMENT );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void requestSendingOfSegment()
    {
        IpEvent event;
        event.constructFromEvent( lastEvent() );

        IpEvent tcpSendSegment( IEvent::TCP_SEND_SEGMENT, event.getIp(), event.getPacketPayload() );

        clearEventVectors();
        _requested.append( tcpSendSegment );
        _blocked.append( IEvent::TCP_OUTGOING_SEGMENT );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }
};

#endif // __OutgoingSegmentHandler_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
