#ifndef __RawSocketReceiver_h__
#define __RawSocketReceiver_h__

#include "BThread.h"
#include "Event.h"
#include "IEvent.h"
#include "IpEvent.h"
#include "IpHeader.h"
#include "RawSocket.h"
#include "TcpEvent.h"
#include <arpa/inet.h>

class RawSocketReceiver : public BThread
{
public:
    RawSocketReceiver( RawSocket &rawSocket ) : BThread( BThread::NON_BLOCKING_THREAD ), _rawSocket( rawSocket )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                receiveData();
                generateDataEvents();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: RawSocketReceiver" );
            throw e;
        }
    }

private:
    RawSocket &_rawSocket;
    HeapData _data;
    struct sockaddr_in _address;
    unsigned _indexInData;

    void receiveData()
    {
        memset( &_address, 0, sizeof(_address) );
        _rawSocket.receive( _data, _address );
    }

    void generateDataEvents()
    {
        _indexInData = 0;

        while ( _indexInData < _data.size() )
        {
            handleSinglePacket();
        }
    }

    void handleSinglePacket()
    {
        IpHeader ipHeader;
        ipHeader.unserialize( ConstSimpleData( _data.asChar() + _indexInData, _data.size() - _indexInData ) );

        if ( !tcpPacket( ipHeader ) )
        {
            return;
        }

        clearEventVectors();

        IpEvent incomingSegment( IEvent::TCP_SEGMENT_RECEIVED,
                                extractIp( ipHeader ),
                                ConstSimpleData( _data.asChar() + ipHeader.size(), ipHeader._mandatoryPart._totalLength - ipHeader.size() ) );

        _requested.append( incomingSegment );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
        _indexInData += ipHeader._mandatoryPart._totalLength;
    }

    static bool tcpPacket( const IpHeader &ipHeader )
    {
        enum {
            TCP_PROTOCOL_IDENTIFIER = 6,
        };

        return ipHeader._mandatoryPart._protocol == TCP_PROTOCOL_IDENTIFIER;
    }

    String extractIp( const IpHeader &ipHeader )
    {
        struct in_addr ip_addr;
        ip_addr.s_addr = ntohl( ipHeader._mandatoryPart._sourceIp );
        return inet_ntoa( ip_addr );
    }
};

#endif // __RawSocketReceiver_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
