#ifndef __RawSocketSender_h__
#define __RawSocketSender_h__

#include "BThread.h"
#include "Event.h"
#include "IEvent.h"
#include "RawSocket.h"
#include "ServerConfiguration.h"

class RawSocketSender : public BThread
{
public:
    RawSocketSender( RawSocket &rawSocket ) : BThread( BThread::REGULAR_THREAD ), _rawSocket( rawSocket )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForSendRequests();
                sendDataOnSocket();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: RawSocketSender" );
            throw e;
        }
    }

private:
    RawSocket &_rawSocket;

    void waitForSendRequests()
    {
        clearEventVectors();
        _watched.append( IEvent::TCP_SEND_SEGMENT );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void sendDataOnSocket()
    {
        IpEvent event;
        event.constructFromEvent( lastEvent() );

        _rawSocket.send( event.getPacketPayload(), event.getIp() );
    }
};

#endif // __RawSocketSender_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
