#ifndef __Retransmitter_h__
#define __Retransmitter_h__

#include "BThread.h"
#include "ConnectionIdentifier.h"
#include "IEvent.h"
#include "PeriodicSender.h"
#include "TcpSegment.h"

class Retransmitter : public BThread
{
public:
    Retransmitter() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForPackets();
                handlePacket();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: Retransmitter" );
            throw e;
        }
    }

private:
    void handlePacket()
    {
        IpEvent ipEvent;
        ipEvent.constructFromEvent( lastEvent() );

        TcpSegment tcpSegment;
        tcpSegment.unserialize( ipEvent.getPacketPayload() );

        if ( !tcpSegment.header().getPush() )
        {
            return;
        }

        ConnectionIdentifier connectionIdentifier( ipEvent.getIp(), tcpSegment.header().getDestinationPort() );

        ( new PeriodicSender( connectionIdentifier,
                              ipEvent.getPacketPayload(),
                              tcpSegment.header().getSequenceNumber() + tcpSegment._payload.size() ) )->run();

        // log( Stringf( "Storing packet. Ip: %s, port: %u, sequence number: 0x%x", tcpEvent.getIp().ascii(), tcpEvent.getPort(), tcpSegment.header().getSequenceNumber() ) );
    }

    void waitForPackets()
    {
        clearEventVectors();
        _watched.append( IEvent::TCP_OUTGOING_SEGMENT );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void log( String message )
    {
        printf( "Retransmitter: %s\n", message.ascii() );
    }
};

#endif // __Retransmitter_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
