#ifndef __TcpKeepAliveMaster_h__
#define __TcpKeepAliveMaster_h__

#include "BThread.h"
#include "ITcpKeepAliveMaster.h"
#include "ServerConfiguration.h"
#include "TcpKeepAliveSender.h"

class TcpKeepAliveMaster : public BThread, public ITcpKeepAliveMaster
{
public:
    TcpKeepAliveMaster() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForSynsAndFins();
                constructConnectionIdentifier();

                switch ( lastEvent().type() )
                {
                case IEvent::TCP_SYN_REQUEST:
                    handleSyn();
                    break;

                case IEvent::TCP_FIN_REQUEST:
                case IEvent::TCP_SEND_FIN:
                    handleFin();
                    break;

                default:
                    break;
                }
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: TcpKeepAliveMaster" );
            throw e;
        }
    }

    bool shouldTerminate( ConnectionIdentifier connectionIdentifier )
    {
        if ( _markedForTermination.exists( connectionIdentifier ) )
        {
            cleanUp( connectionIdentifier );
            return true;
        }

        return false;
    }

private:
    ConnectionIdentifier _connectionIdentifier;
    Map<ConnectionIdentifier, BThread *> _keepAliveSenders;
    UniqueVector<ConnectionIdentifier> _markedForTermination;

    void waitForSynsAndFins()
    {
        clearEventVectors();
        _watched.append( IEvent::TCP_SYN_REQUEST );
        _watched.append( IEvent::TCP_FIN_REQUEST );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void constructConnectionIdentifier()
    {
        TcpEvent tcpEvent;
        tcpEvent.constructFromEvent( lastEvent() );

        _connectionIdentifier = ConnectionIdentifier( tcpEvent.getIp(), tcpEvent.getPort() );
    }

    void handleSyn()
    {
        if ( !keepAliveSenderExists() )
        {
            createKeepAliveSender();
        }
    }

    bool keepAliveSenderExists()
    {
        return _keepAliveSenders.exists( _connectionIdentifier );
    }

    void createKeepAliveSender()
    {
        ( _keepAliveSenders[_connectionIdentifier] = new TcpKeepAliveSender( _connectionIdentifier, *this ) )->run();
    }

    void handleFin()
    {
        if ( _keepAliveSenders.exists( _connectionIdentifier ) )
        {
            _markedForTermination.append( _connectionIdentifier );
        }
    }

    void cleanUp( ConnectionIdentifier &connectionIdentifier )
    {
        _markedForTermination.erase( connectionIdentifier );
        _keepAliveSenders.erase( connectionIdentifier );
    }

    void log( String message )
    {
        printf( "\t\tTcpKeepAliveMaster: %s\n", message.ascii() );
        fflush( 0 );
    }
};

#endif // __TcpKeepAliveMaster\_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
