#ifndef __TcpServer_h__
#define __TcpServer_h__

#include "AckMaster.h"
#include "BProgram.h"
#include "BridgeToHttp.h"
#include "BridgeToTcp.h"
#include "CustomizedOracleChooser.h"
#include "HttpCgiRunner.h"
#include "HttpDispatcher.h"
#include "HttpErrorHandler.h"
#include "HttpFoundHandler.h"
#include "HttpGetHandler.h"
#include "Logger.h"
#include "Oracle.h"
#include "OutgoingSegmentHandler.h"
#include "RawSocketReceiver.h"
#include "RawSocketSender.h"
#include "Retransmitter.h"
#include "SegmentSorter.h"
#include "SequenceMaster.h"
#include "TcpAckSender.h"
#include "TcpDispatcher.h"
#include "TcpEnsureActiveConnection.h"
#include "TcpFinHandler.h"
#include "TcpFinSender.h"
#include "TcpKeepAliveMaster.h"
#include "TcpPushHandler.h"
#include "TcpPushHandler.h"
#include "TcpPushSender.h"
#include "TcpSynHandler.h"

class TcpServer
{
public:
    TcpServer() : _rawSocket( ServerConfiguration::SERVER_IP ), _rawSocketReceiver( _rawSocket ), _rawSocketSender( _rawSocket )
    {
    }

    static void start()
    {
        TcpServer().run();
    }

    void run()
    {
        setOracleChooser();
        runEngine();

        try
        {
            initializeMainSocket();
            runProgram();
            sleepForever();
        }
        catch ( const Error &e )
        {
            reportError( e );
        }
    }

private:
    BProgram _bProgram;
    RawSocket _rawSocket;

    AckMaster _ackMaster;
    OutgoingSegmentHandler _outgoingSegmentHandler;
    RawSocketReceiver _rawSocketReceiver;
    RawSocketSender _rawSocketSender;
    SequenceMaster _sequenceMaster;
    TcpAckSender _tcpAckSender;
    TcpDispatcher _tcpDispatcher;
    TcpEnsureActiveConnection _tcpEnsureActiveConnection;
    TcpFinHandler _tcpFinHandler;
    TcpFinSender _tcpFinSender;
    TcpPushHandler _tcpPushHandler;
    TcpPushSender _tcpPushSender;
    TcpSynHandler _tcpSynHandler;

    BridgeToTcp _bridgeToTcp;
    BridgeToHttp _bridgeToHttp;

    SegmentSorter _segmentSorter;

    HttpDispatcher _httpDispatcher;
    HttpGetHandler _httpGetHandler;
    HttpCgiRunner _httpCgiRunner;
    HttpErrorHandler _httpErrorHandler;
    HttpFoundHandler _httpFoundHandler;

    // HttpFileReceiver _httpFileReceiver;

    TcpKeepAliveMaster _tcpKeepAliveMaster;
    Retransmitter _retransmitter;

    Logger _logger;

    CustomizedOracleChooser _customizedOracleChooser;

    void runEngine()
    {
        _bProgram.runEngine();
    }

    void initializeMainSocket()
    {
        _rawSocket.activate();
    }

    void runProgram()
    {
        _bProgram.addThread( _ackMaster );
        _bProgram.addThread( _outgoingSegmentHandler );
        _bProgram.addThread( _rawSocketReceiver );
        _bProgram.addThread( _rawSocketSender );
        _bProgram.addThread( _sequenceMaster );
        _bProgram.addThread( _tcpAckSender );
        _bProgram.addThread( _tcpDispatcher );
        _bProgram.addThread( _tcpEnsureActiveConnection );
        _bProgram.addThread( _tcpFinHandler );
        _bProgram.addThread( _tcpFinSender );
        _bProgram.addThread( _tcpPushHandler );
        _bProgram.addThread( _tcpPushSender );
        _bProgram.addThread( _tcpSynHandler );

        _bProgram.addThread( _segmentSorter );

        _bProgram.addThread( _bridgeToTcp );
        _bProgram.addThread( _bridgeToHttp );

        _bProgram.addThread( _httpDispatcher );
        _bProgram.addThread( _httpGetHandler );
        _bProgram.addThread( _httpCgiRunner );
        _bProgram.addThread( _httpErrorHandler );
        _bProgram.addThread( _httpFoundHandler );

        // _bProgram.addThread( _httpFileReceiver );

        _bProgram.addThread( _tcpKeepAliveMaster );
        _bProgram.addThread( _retransmitter );

        _bProgram.addThread( _logger );

        _bProgram.runProgram();
    }

    void sleepForever()
    {
        enum {
            SLEEP_INTERVAL = 1000,
        };

        while ( true )
        {
            sleep( SLEEP_INTERVAL );
        }
    }

    void reportError( const Error &e )
    {
        printf( "Main Thread: Error caught. Code: %u. Errno: %i\n", e.code(), e.errno() );
        fflush( 0 );
    }

    void setOracleChooser()
    {
        Oracle::setChooser( &_customizedOracleChooser );
    }
};

#endif // __TcpServer_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
